# rxscrite/parser.py

from .tokens import (
    TOKEN_INTEGER, TOKEN_FLOAT, TOKEN_PLUS, TOKEN_MINUS, TOKEN_MULTIPLY,
    TOKEN_DIVIDE, TOKEN_POWER, TOKEN_LPAREN, TOKEN_RPAREN, TOKEN_EOF,
    TOKEN_IDENTIFIER, TOKEN_KEYWORD, TOKEN_ASSIGN, TOKEN_COMMA, TOKEN_NEWLINE,
    TOKEN_STRING,

    # New Operator Tokens
    TOKEN_EE, TOKEN_NE, TOKEN_LT, TOKEN_GT, TOKEN_LTE, TOKEN_GTE,
    TOKEN_LOGICAL_AND, TOKEN_LOGICAL_OR, TOKEN_LOGICAL_NOT, TOKEN_MEM_IN,
    
    TOKEN_MODULO, TOKEN_INT_DIVIDE,
    TOKEN_PLUS_ASSIGN, TOKEN_MINUS_ASSIGN, TOKEN_MUL_ASSIGN, TOKEN_DIV_ASSIGN,
    TOKEN_MOD_ASSIGN, TOKEN_POW_ASSIGN, TOKEN_INT_DIV_ASSIGN
)
from .nodes import (
    NumberNode, BinaryOpNode, UnaryOpNode, VariableAssignNode,
    VariableAccessNode, FunctionCallNode, ProgramNode, IdentifierNode,
    StringNode, TruthNode, FalsityNode, EmptyNode, NoOpNode,
    CompoundAssignNode 
)
from .errors import RxParserError

class Parser:
    def __init__(self, tokens, filename="<stdin>"):
        self.tokens = tokens
        self.filename = filename
        self.token_idx = -1
        self.current_token = None
        self.advance()

    def advance(self):
        self.token_idx += 1
        if self.token_idx < len(self.tokens):
            self.current_token = self.tokens[self.token_idx]
        else:
            last_tok = self.tokens[-1] if self.tokens else None
            line = last_tok.line if last_tok else (self.current_token.line if self.current_token else 1)
            col = last_tok.column if last_tok else (self.current_token.column if self.current_token else 1)
            self.current_token = Token(TOKEN_EOF, None, line=line, column=col)
        # print(f"Advanced to: {self.current_token}") # General advance debug
        return self.current_token
    
    def _current_line_col(self): # Helper for error reporting if current_token is None
        if self.current_token and self.current_token.type != TOKEN_EOF:
            return self.current_token.line, self.current_token.column
        # Try to get from previous token if at EOF or current_token is None
        if self.token_idx > 0 and self.token_idx -1 < len(self.tokens) :
             prev_token = self.tokens[self.token_idx-1]
             return prev_token.line, prev_token.column
        return 1,1 # Fallback

    def peek(self):
        if self.token_idx + 1 < len(self.tokens):
            return self.tokens[self.token_idx + 1]
        l,c = self._current_line_col()
        return Token(TOKEN_EOF, None, line=l, column=c) 

    def error(self, expected_token_type=None, message=""):
        line, column = self._current_line_col()
        if self.current_token and self.current_token.type != TOKEN_EOF : # Prefer current token's pos if valid
            line = self.current_token.line
            column = self.current_token.column

        if not message:
            if expected_token_type and self.current_token:
                message = f"Expected {expected_token_type}, but got {self.current_token.type} ('{self.current_token.value}')"
            elif self.current_token:
                message = f"Unexpected token: {self.current_token}"
            else:
                message = "Unexpected end of file (or token is None)."
        
        raise RxParserError(message, line=line, column=column)

    def eat(self, token_type, token_value=None):
        if self.current_token is None or self.current_token.type == TOKEN_EOF:
            l,c = self._current_line_col()
            raise RxParserError(f"Unexpected EOF. Expected {token_type}", line=l, column=c)
        
        if self.current_token.type == token_type:
            if token_value is not None and self.current_token.value != token_value:
                self.error(expected_token_type, f"Expected token value '{token_value}' but got '{self.current_token.value}'")
            eaten_token = self.current_token
            self.advance()
            return eaten_token
        else:
            self.error(expected_token_type, f"Expected token type {token_type} but got {self.current_token.type} ('{self.current_token.value}')")


    def program(self):
        prog_node = ProgramNode()
        while self.current_token and self.current_token.type == TOKEN_NEWLINE:
            self.eat(TOKEN_NEWLINE)

        while self.current_token and self.current_token.type != TOKEN_EOF:
            if self.current_token.type == TOKEN_NEWLINE: 
                self.eat(TOKEN_NEWLINE)
                continue
            
            # print(f"Program loop: current token before statement(): {self.current_token}") # Debug
            stmt = self.statement()
            # print(f"Program loop: current token after statement(): {self.current_token}") # Debug
            
            if stmt and not isinstance(stmt, NoOpNode):
                prog_node.statements.append(stmt)

            if self.current_token.type != TOKEN_EOF:
                if self.current_token.type == TOKEN_NEWLINE:
                    while self.current_token and self.current_token.type == TOKEN_NEWLINE:
                        self.eat(TOKEN_NEWLINE)
                else:
                    # print(f"ERROR Point in program(): current_token is {self.current_token}") # Debug
                    self.error(TOKEN_NEWLINE, "Expected newline or EOF after statement.")
            
            while self.current_token and self.current_token.type == TOKEN_NEWLINE: # Consume any further blank lines
                self.eat(TOKEN_NEWLINE)
        return prog_node

    def statement(self):
        if self.current_token.type == TOKEN_IDENTIFIER:
            peeked_token = self.peek()
            if peeked_token.type in (TOKEN_ASSIGN, TOKEN_PLUS_ASSIGN, TOKEN_MINUS_ASSIGN,
                                     TOKEN_MUL_ASSIGN, TOKEN_DIV_ASSIGN, TOKEN_MOD_ASSIGN,
                                     TOKEN_POW_ASSIGN, TOKEN_INT_DIV_ASSIGN):
                return self.assignment_statement()
        
        return self.expression()


    def assignment_statement(self):
        var_name_token = self.eat(TOKEN_IDENTIFIER)
        assign_op_token = self.current_token
        
        if assign_op_token.type == TOKEN_ASSIGN:
            self.eat(TOKEN_ASSIGN)
            value_node = self.expression()
            return VariableAssignNode(var_name_token, value_node)
        elif assign_op_token.type in (TOKEN_PLUS_ASSIGN, TOKEN_MINUS_ASSIGN, TOKEN_MUL_ASSIGN, 
                                      TOKEN_DIV_ASSIGN, TOKEN_MOD_ASSIGN, TOKEN_POW_ASSIGN,
                                      TOKEN_INT_DIV_ASSIGN):
            self.eat(assign_op_token.type)
            value_node = self.expression()
            return CompoundAssignNode(var_name_token, assign_op_token, value_node)
        else:
            self.error(message=f"Expected assignment operator (=, +=, etc.) but got {assign_op_token.type}")
            return NoOpNode() 

    def expression(self):
        return self.logical_or_expression()

    def logical_or_expression(self):
        node = self.logical_and_expression()
        while self.current_token.type == TOKEN_LOGICAL_OR:
            op_token = self.eat(TOKEN_LOGICAL_OR)
            node = BinaryOpNode(left_node=node, op_token=op_token, right_node=self.logical_and_expression())
        return node

    def logical_and_expression(self):
        node = self.not_expression() # Changed from comparison_expression
        while self.current_token.type == TOKEN_LOGICAL_AND:
            op_token = self.eat(TOKEN_LOGICAL_AND)
            node = BinaryOpNode(left_node=node, op_token=op_token, right_node=self.not_expression())
        return node

    def not_expression(self):
        if self.current_token.type == TOKEN_LOGICAL_NOT:
            op_token = self.eat(TOKEN_LOGICAL_NOT)
            node = UnaryOpNode(op_token, self.not_expression()) 
            return node
        else:
            return self.comparison_expression()

    def comparison_expression(self):
        node = self.membership_expression()
        while self.current_token.type in (TOKEN_EE, TOKEN_NE, TOKEN_LT, TOKEN_GT, TOKEN_LTE, TOKEN_GTE):
            op_token = self.current_token
            self.eat(op_token.type)
            node = BinaryOpNode(left_node=node, op_token=op_token, right_node=self.membership_expression())
        return node

    def membership_expression(self):
        node = self.arith_expression()
        # Handle 'in' and 'not in'
        # 'not in' is parsed as: not (expr in expr) due to not_expression precedence
        if self.current_token.type == TOKEN_MEM_IN:
            op_token = self.eat(TOKEN_MEM_IN)
            node = BinaryOpNode(left_node=node, op_token=op_token, right_node=self.arith_expression())
        return node

    def arith_expression(self):
        node = self.term()
        while self.current_token.type in (TOKEN_PLUS, TOKEN_MINUS):
            op_token = self.current_token
            self.eat(op_token.type)
            node = BinaryOpNode(left_node=node, op_token=op_token, right_node=self.term())
        return node

    def term(self):
        node = self.factor()
        while self.current_token.type in (TOKEN_MULTIPLY, TOKEN_DIVIDE, TOKEN_INT_DIVIDE, TOKEN_MODULO):
            op_token = self.current_token
            self.eat(op_token.type)
            node = BinaryOpNode(left_node=node, op_token=op_token, right_node=self.factor())
        return node

    def factor(self):
        token = self.current_token
        if token.type == TOKEN_PLUS:
            op_token = self.eat(TOKEN_PLUS)
            return UnaryOpNode(op_token, self.factor())
        elif token.type == TOKEN_MINUS:
            op_token = self.eat(TOKEN_MINUS)
            return UnaryOpNode(op_token, self.factor())
        return self.power()

    def power(self):
        node = self.call() 
        while self.current_token.type == TOKEN_POWER:
            op_token = self.eat(TOKEN_POWER)
            node = BinaryOpNode(left_node=node, op_token=op_token, right_node=self.factor())
        return node

    def call(self):
        node = self.atom()
        while self.current_token.type == TOKEN_LPAREN:
            if not isinstance(node, (IdentifierNode, VariableAccessNode)): # Check if atom is callable
                 self.error(message=f"'{node}' is not callable.")

            self.eat(TOKEN_LPAREN)
            arg_nodes = []
            if self.current_token.type != TOKEN_RPAREN:
                arg_nodes.append(self.expression())
                while self.current_token.type == TOKEN_COMMA:
                    self.eat(TOKEN_COMMA)
                    arg_nodes.append(self.expression())
            self.eat(TOKEN_RPAREN)
            
            func_name_node_for_call = node
            if isinstance(node, VariableAccessNode): # Convert VarAccess to Identifier for func name
                func_name_node_for_call = IdentifierNode(node.var_name_token)
            
            node = FunctionCallNode(func_name_node_for_call, arg_nodes)
        return node

    def atom(self):
        token = self.current_token

        if token.type == TOKEN_INTEGER:
            self.eat(TOKEN_INTEGER)
            # ---- DEBUG PRINT ----
            # print(f"DEBUG (atom - INTEGER): Ate {token}, current_token is now {self.current_token}")
            # ---- END DEBUG PRINT ----
            return NumberNode(token)
        elif token.type == TOKEN_FLOAT:
            self.eat(TOKEN_FLOAT)
            # print(f"DEBUG (atom - FLOAT): Ate {token}, current_token is now {self.current_token}")
            return NumberNode(token)
        elif token.type == TOKEN_STRING:
            self.eat(TOKEN_STRING)
            return StringNode(token)
        elif token.type == TOKEN_IDENTIFIER:
            self.eat(TOKEN_IDENTIFIER)
            return VariableAccessNode(token)
        elif token.type == TOKEN_KEYWORD:
            keyword_val = token.value
            if keyword_val == 'Truth':
                self.eat(TOKEN_KEYWORD)
                return TruthNode(token)
            elif keyword_val == 'Falsity':
                self.eat(TOKEN_KEYWORD)
                return FalsityNode(token)
            elif keyword_val == 'Empty':
                self.eat(TOKEN_KEYWORD)
                return EmptyNode(token)
            elif keyword_val in ('display', 'ask'):
                self.eat(TOKEN_KEYWORD)
                return IdentifierNode(token)
            else:
                self.error(message=f"Unexpected keyword '{keyword_val}' as an atom.")
        elif token.type == TOKEN_LPAREN:
            self.eat(TOKEN_LPAREN)
            node = self.expression()
            self.eat(TOKEN_RPAREN)
            return node
        
        self.error(message=f"Unexpected token in atom(): {token}")
        return NoOpNode() 

    def parse(self):
        # print("Starting parse...") # Debug
        # for t in self.tokens: print(t) # Debug: Print all tokens parser received
        if not self.tokens or (len(self.tokens) == 1 and self.tokens[0].type == TOKEN_EOF):
            # print("Parser: Empty or EOF-only tokens, returning empty ProgramNode.") # Debug
            return ProgramNode()
            
        ast = self.program()
        # print(f"Parse complete. Final current_token: {self.current_token}") # Debug
        if self.current_token and self.current_token.type != TOKEN_EOF:
            self.error(TOKEN_EOF, f"Expected EOF but found {self.current_token.type} ('{self.current_token.value}') after parsing.")
        return ast




# rxscrite/nodes.py

class ASTNode:
    """Base class for all AST nodes."""
    def __init__(self, token=None): # Optional token for position tracking
        self.token = token
        # Safely access line and column, providing None if token or attributes are missing
        self.line = token.line if token and hasattr(token, 'line') else None
        self.column = token.column if token and hasattr(token, 'column') else None

    def __repr__(self):
        return f"<{self.__class__.__name__}>"

# --- Literal and Identifier Nodes ---
class NumberNode(ASTNode):
    """Node representing a numeric literal (integer or float)."""
    def __init__(self, token):
        super().__init__(token)
        self.value = token.value # The actual number (int or float)

    def __repr__(self):
        return f"NumberNode({self.value})"

class StringNode(ASTNode):
    """Node representing a string literal."""
    def __init__(self, token):
        super().__init__(token)
        self.value = token.value # The actual string value

    def __repr__(self):
        return f"StringNode({repr(self.value)})"

class IdentifierNode(ASTNode):
    """Node representing an identifier (e.g., variable name, function name)."""
    def __init__(self, token):
        super().__init__(token)
        self.value = token.value # The name of the identifier

    def __repr__(self):
        return f"IdentifierNode({self.value})"

# --- Boolean and Null-equivalent Nodes ---
class TruthNode(ASTNode):
    """Node representing the 'Truth' keyword (boolean True)."""
    def __init__(self, token):
        super().__init__(token)
        self.value = True
    def __repr__(self):
        return f"TruthNode({self.value})"

class FalsityNode(ASTNode):
    """Node representing the 'Falsity' keyword (boolean False)."""
    def __init__(self, token):
        super().__init__(token)
        self.value = False
    def __repr__(self):
        return f"FalsityNode({self.value})"

class EmptyNode(ASTNode):
    """Node representing the 'Empty' keyword (similar to None)."""
    def __init__(self, token):
        super().__init__(token)
        self.value = None # Or a special EmptyType object if needed later
    def __repr__(self):
        return f"EmptyNode({self.value})"

# --- Operator Nodes ---
class UnaryOpNode(ASTNode):
    """Node representing a unary operation (e.g., -5, not True)."""
    def __init__(self, op_token, node):
        super().__init__(op_token) # op_token is like TOKEN_MINUS, TOKEN_LOGICAL_NOT
        self.op_token = op_token
        self.node = node # The node to operate on (e.g., NumberNode, ExpressionNode)

    def __repr__(self):
        return f"UnaryOpNode(op={self.op_token.type}, value={self.node})"

class BinaryOpNode(ASTNode):
    """Node representing a binary operation (e.g., a + b, x == y)."""
    def __init__(self, left_node, op_token, right_node):
        super().__init__(op_token) # op_token is like TOKEN_PLUS, TOKEN_EE
        self.left_node = left_node
        self.op_token = op_token
        self.right_node = right_node

    def __repr__(self):
        return f"BinaryOpNode(left={self.left_node}, op={self.op_token.type}, right={self.right_node})"

# --- Assignment Nodes ---
class VariableAssignNode(ASTNode):
    """Node representing a simple variable assignment (e.g., x = 10)."""
    def __init__(self, var_name_token, value_node): # var_name_token is an IDENTIFIER token
        super().__init__(var_name_token)
        self.var_name_token = var_name_token
        self.value_node = value_node # Node representing the value being assigned

    def __repr__(self):
        return f"VariableAssignNode(var_name='{self.var_name_token.value}', value={self.value_node})"

class CompoundAssignNode(ASTNode): # New Node
    """Node representing a compound variable assignment (e.g., x += 10)."""
    def __init__(self, var_name_token, op_token, value_node):
        super().__init__(var_name_token) # Position from variable name
        self.var_name_token = var_name_token # IDENTIFIER token for the variable
        self.op_token = op_token # The compound assignment token (e.g., TOKEN_PLUS_ASSIGN)
        self.value_node = value_node # Node representing the value on the right

    def __repr__(self):
        return f"CompoundAssignNode(var_name='{self.var_name_token.value}', op='{self.op_token.value}', value={self.value_node})"

# --- Variable Access and Function Call ---
class VariableAccessNode(ASTNode):
    """Node representing access to a variable's value."""
    def __init__(self, var_name_token): # var_name_token is an IDENTIFIER token
        super().__init__(var_name_token)
        self.var_name_token = var_name_token

    def __repr__(self):
        return f"VariableAccessNode(var_name='{self.var_name_token.value}')"

class FunctionCallNode(ASTNode):
    """Node representing a function call."""
    def __init__(self, func_name_node, arg_nodes): # func_name_node is often an IdentifierNode
        # Try to get a representative token for line/col info, usually from the function name
        rep_token = None
        if isinstance(func_name_node, ASTNode) and func_name_node.token:
            rep_token = func_name_node.token
        super().__init__(rep_token)
        
        self.func_name_node = func_name_node # Node representing the function name (e.g., IdentifierNode)
        self.arg_nodes = arg_nodes # List of nodes representing arguments

    def __repr__(self):
        return f"FunctionCallNode(func_name={self.func_name_node}, args_count={len(self.arg_nodes)})"

# --- Program Structure and Control Flow (Placeholders / Basic) ---
class ProgramNode(ASTNode):
    """Node representing a whole program or a block of statements."""
    def __init__(self):
        super().__init__() # No specific token for the whole program
        self.statements = [] # List of statement nodes

    def __repr__(self):
        return f"ProgramNode(statements_count={len(self.statements)})"

class NoOpNode(ASTNode): # New Node
    """Node representing no operation, useful for empty parts or placeholders."""
    def __init__(self, token=None): # token is optional
        super().__init__(token)
    def __repr__(self):
        return "NoOpNode"

# --- Placeholder Nodes for Future Language Features ---
# These can be expanded upon when we implement these features.

class IfNode(ASTNode): # For 'check' statement
    """Node representing an if-elif-else conditional structure."""
    def __init__(self, conditions_and_blocks, else_block):
        # conditions_and_blocks is a list of tuples: (condition_node, block_node)
        # else_block is a block_node or None
        rep_token = None
        if conditions_and_blocks and conditions_and_blocks[0] and conditions_and_blocks[0][0]:
             if hasattr(conditions_and_blocks[0][0], 'token'):
                 rep_token = conditions_and_blocks[0][0].token
        super().__init__(rep_token)
        self.conditions_and_blocks = conditions_and_blocks
        self.else_block = else_block
    # __repr__ can be added for better debugging views

class ForLoopNode(ASTNode): # For 'loop' statement
    """Node representing a for-loop structure."""
    def __init__(self, var_name_token, iterable_node, body_node, else_node=None):
        super().__init__(var_name_token)
        self.var_name_token = var_name_token
        self.iterable_node = iterable_node
        self.body_node = body_node # This would typically be a ProgramNode (block of statements)
        self.else_node = else_node # Optional 'otherwise' block
    # __repr__ can be added

class WhileLoopNode(ASTNode): # For 'repeat_while' statement
    """Node representing a while-loop structure."""
    def __init__(self, condition_node, body_node, else_node=None):
        rep_token = condition_node.token if condition_node and hasattr(condition_node, 'token') else None
        super().__init__(rep_token)
        self.condition_node = condition_node
        self.body_node = body_node # This would typically be a ProgramNode
        self.else_node = else_node # Optional 'otherwise' block
    # __repr__ can be added

class FunctionDefNode(ASTNode): # For 'func' statement
    """Node representing a user-defined function."""
    def __init__(self, func_name_token, param_name_tokens, body_node, return_type_token=None):
        super().__init__(func_name_token)
        self.func_name_token = func_name_token
        self.param_name_tokens = param_name_tokens # List of IDENTIFIER tokens
        self.body_node = body_node # This would typically be a ProgramNode
        # self.return_type_token = return_type_token # For optional type hinting later
    # __repr__ can be added



# rxscrite/tokens.py

# Token types
TOKEN_EOF = 'EOF'

# Data types
TOKEN_INTEGER = 'INTEGER'
TOKEN_FLOAT = 'FLOAT'
TOKEN_STRING = 'STRING'
TOKEN_IDENTIFIER = 'IDENTIFIER'

# General Keywords (for control flow, definitions, etc., not operators)
TOKEN_KEYWORD = 'KEYWORD'

# Operators
TOKEN_PLUS = 'PLUS'            # +
TOKEN_MINUS = 'MINUS'           # -
TOKEN_MULTIPLY = 'MULTIPLY'        # *
TOKEN_DIVIDE = 'DIVIDE'           # /
TOKEN_POWER = 'POWER'           # ^
TOKEN_MODULO = 'MODULO'         # % (New)
TOKEN_INT_DIVIDE = 'INT_DIVIDE'   # // (New)

# Assignment
TOKEN_ASSIGN = 'ASSIGN'          # =
# Compound Assignment Operators (New)
TOKEN_PLUS_ASSIGN = 'PLUS_ASSIGN'     # +=
TOKEN_MINUS_ASSIGN = 'MINUS_ASSIGN'    # -=
TOKEN_MUL_ASSIGN = 'MUL_ASSIGN'       # *=
TOKEN_DIV_ASSIGN = 'DIV_ASSIGN'       # /=
TOKEN_MOD_ASSIGN = 'MOD_ASSIGN'       # %=
TOKEN_POW_ASSIGN = 'POW_ASSIGN'       # ^=
TOKEN_INT_DIV_ASSIGN = 'INT_DIV_ASSIGN' # //=

# Delimiters
TOKEN_LPAREN = 'LPAREN'        # (
TOKEN_RPAREN = 'RPAREN'        # )
TOKEN_COMMA = 'COMMA'           # ,
TOKEN_NEWLINE = 'NEWLINE'        # \n

# Comparison Operators (New - Python Style)
TOKEN_EE = 'EE'                # == (Equal Equal)
TOKEN_NE = 'NE'                # != (Not Equal)
TOKEN_LT = 'LT'                # <  (Less Than)
TOKEN_GT = 'GT'                # >  (Greater Than)
TOKEN_LTE = 'LTE'               # <= (Less Than or Equal)
TOKEN_GTE = 'GTE'               # >= (Greater Than or Equal)

# Logical Operators (New - Python Style Keywords, treated as distinct tokens)
TOKEN_LOGICAL_AND = 'LOGICAL_AND' # 'and'
TOKEN_LOGICAL_OR = 'LOGICAL_OR'  # 'or'
TOKEN_LOGICAL_NOT = 'LOGICAL_NOT' # 'not' (unary)

# Membership Operators (New - Python Style Keywords, treated as distinct tokens)
TOKEN_MEM_IN = 'MEM_IN'          # 'in'
TOKEN_MEM_NOT_IN = 'MEM_NOT_IN'  # 'not in' (will be lexed as two tokens: NOT then IN, or a single combined token)
                                 # For simplicity, let's aim for 'not in' as a distinct concept,
                                 # but lexing it might involve peeking.
                                 # For now, let's define distinct tokens for 'in' and 'not'.
                                 # The parser will handle 'not in' logic.

# Reserved Keywords (for language constructs, not operators anymore)
# We've removed: is_equal_to, greater_than, both, either, negate, inside etc.
# 'and', 'or', 'not', 'in' are now specific token types above.
RESERVED_KEYWORDS = {
    'display': TOKEN_KEYWORD,
    'ask': TOKEN_KEYWORD,
    'Truth': TOKEN_KEYWORD,
    'Falsity': TOKEN_KEYWORD,
    'Empty': TOKEN_KEYWORD,
    'check': TOKEN_KEYWORD,
    'otherwise': TOKEN_KEYWORD,
    'orcheck': TOKEN_KEYWORD,
    'loop': TOKEN_KEYWORD,
    'repeat_while': TOKEN_KEYWORD,
    'func': TOKEN_KEYWORD,
    'blueprint': TOKEN_KEYWORD,
    'giveback': TOKEN_KEYWORD,
    'constructor': TOKEN_KEYWORD,
    'skip': TOKEN_KEYWORD,
    'exit_loop': TOKEN_KEYWORD,
    'next_item': TOKEN_KEYWORD,
    'use': TOKEN_KEYWORD,
    'from_module': TOKEN_KEYWORD,
    'alias': TOKEN_KEYWORD,
    'attempt': TOKEN_KEYWORD,
    'on_error': TOKEN_KEYWORD,
    'ensure': TOKEN_KEYWORD,
    'alert': TOKEN_KEYWORD,
    'verify': TOKEN_KEYWORD,
    'remove': TOKEN_KEYWORD,
    # Python's 'is' and 'is not' (identity) could be added later if needed
    # For now, 'in' is TOKEN_MEM_IN. 'not' is TOKEN_LOGICAL_NOT.
    # The parser will combine 'not' and 'in' if it sees them together for 'not in'.
}

# Special map for keyword-like operators, distinct from general identifiers
# This helps the lexer identify 'and', 'or', 'not', 'in' as specific operator tokens
# rather than TOKEN_IDENTIFIER or general TOKEN_KEYWORD.
KEYWORD_OPERATORS = {
    'and': TOKEN_LOGICAL_AND,
    'or': TOKEN_LOGICAL_OR,
    'not': TOKEN_LOGICAL_NOT,
    'in': TOKEN_MEM_IN,
}


class Token:
    def __init__(self, type, value=None, line=None, column=None):
        self.type = type
        self.value = value
        self.line = line
        self.column = column

    def __str__(self):
        return f"Token({self.type}, {repr(self.value)}" + \
               (f", line={self.line}, col={self.column}" if self.line is not None else "") + ")"

    def __repr__(self):
        return self.__str__()

    def matches(self, type_, value_):
        return self.type == type_ and self.value == value_



# rxscrite/errors.py

class RxError(Exception):
    """Base class for all RxScrite errors."""
    def __init__(self, message, line=None, column=None, details=""):
        super().__init__(message)
        self.message = message
        self.line = line
        self.column = column
        self.details = details # For more context if needed

    def __str__(self):
        loc = ""
        if self.line is not None:
            loc = f" (Line {self.line}"
            if self.column is not None:
                loc += f", Column {self.column}"
            loc += ")"
        return f"{self.__class__.__name__}{loc}: {self.message}" + (f"\nDetails: {self.details}" if self.details else "")

class RxLexerError(RxError):
    """Error during lexical analysis."""
    pass

class RxParserError(RxError):
    """Error during parsing (syntax error)."""
    pass

class RxSemanticError(RxError):
    """Error during semantic analysis (e.g., type mismatch, undefined variable)."""
    pass # We'll use this more when the interpreter/checker is more advanced

class RxRuntimeError(RxError):
    """Error during program execution (interpreter)."""
    pass

# Specific runtime errors (can be expanded from our plan like ValueProblem, TypeProblem)
class RxNameError(RxRuntimeError):
    """Raised when a name (variable, function) is not found."""
    pass

class RxTypeError(RxRuntimeError):
    """Raised when an operation or function is applied to an object of inappropriate type."""
    pass

class RxZeroDivisionError(RxRuntimeError):
    """Raised when division or modulo by zero takes place for integers or floats."""
    pass

# Example of a more specific planned error
class RxValueProblem(RxRuntimeError): # Corresponds to Python's ValueError
    """Raised when a built-in operation or function receives an argument that has the
    right type but an inappropriate value."""
    pass

# You can add more specific error types here as needed, for example:
# class RxImportProblem(RxRuntimeError): pass
# class RxIndexProblem(RxRuntimeError): pass


# rxscrite/lexer.py

from .tokens import (
    Token,
    TOKEN_INTEGER, TOKEN_FLOAT, TOKEN_PLUS, TOKEN_MINUS, TOKEN_MULTIPLY,
    TOKEN_DIVIDE, TOKEN_POWER, TOKEN_LPAREN, TOKEN_RPAREN, TOKEN_EOF,
    TOKEN_IDENTIFIER, TOKEN_KEYWORD, RESERVED_KEYWORDS, TOKEN_ASSIGN,
    TOKEN_COMMA, TOKEN_NEWLINE, TOKEN_STRING,

    # New Operator Tokens
    TOKEN_EE, TOKEN_NE, TOKEN_LT, TOKEN_GT, TOKEN_LTE, TOKEN_GTE,
    TOKEN_LOGICAL_AND, TOKEN_LOGICAL_OR, TOKEN_LOGICAL_NOT, TOKEN_MEM_IN,
    KEYWORD_OPERATORS, # For 'and', 'or', 'not', 'in'

    TOKEN_MODULO, TOKEN_INT_DIVIDE,
    TOKEN_PLUS_ASSIGN, TOKEN_MINUS_ASSIGN, TOKEN_MUL_ASSIGN, TOKEN_DIV_ASSIGN,
    TOKEN_MOD_ASSIGN, TOKEN_POW_ASSIGN, TOKEN_INT_DIV_ASSIGN
)
from .errors import RxLexerError

class Lexer:
    def __init__(self, text, filename="<stdin>"):
        self.text = text
        self.filename = filename
        self.pos = 0
        self.current_char = self.text[self.pos] if self.text else None
        self.line = 1
        self.column = 1

    def advance(self):
        """Advance the 'pos' pointer and set 'current_char'."""
        if self.current_char == '\n':
            self.line += 1
            self.column = 0 # Reset column, next advance will make it 1

        self.pos += 1
        if self.pos > len(self.text) - 1:
            self.current_char = None
        else:
            self.current_char = self.text[self.pos]
        
        self.column +=1

    def peek(self, offset=1):
        """Look ahead at characters without consuming the current one."""
        peek_pos = self.pos + offset
        if peek_pos > len(self.text) - 1:
            return None
        else:
            return self.text[peek_pos]

    def skip_whitespace(self):
        while self.current_char is not None and self.current_char.isspace() and self.current_char != '\n':
            self.advance()

    def skip_comment(self):
        if self.current_char == '#':
            while self.current_char is not None and self.current_char != '\n':
                self.advance()

    def number(self):
        result = ''
        start_line = self.line
        start_column = self.column
        is_float = False

        while self.current_char is not None and self.current_char.isdigit():
            result += self.current_char
            self.advance()

        if self.current_char == '.':
            if self.peek() and self.peek().isdigit(): 
                result += '.'
                self.advance()
                is_float = True
                while self.current_char is not None and self.current_char.isdigit():
                    result += self.current_char
                    self.advance()
            # If dot is not followed by a digit, it's not part of this number.
        
        if is_float:
            # Ensure there are digits after the decimal point if it's marked as float
            if not result.split('.')[1]: 
                 self.error(f"Invalid float format: '{result}' missing digits after decimal point. Use number.0 for whole floats.", line=start_line, column=start_column)
            return Token(TOKEN_FLOAT, float(result), start_line, start_column)
        else:
            return Token(TOKEN_INTEGER, int(result), start_line, start_column)

    def identifier(self):
        result = ''
        start_line = self.line
        start_column = self.column

        if self.current_char is not None and (self.current_char.isalpha() or self.current_char == '_'):
            result += self.current_char
            self.advance()
        else: 
            self.error(f"Identifier cannot start with '{self.current_char}'", line=start_line, column=start_column)
            return Token(TOKEN_EOF, None, self.line, self.column) # Should not be reached if error raises

        while self.current_char is not None and (self.current_char.isalnum() or self.current_char == '_'):
            result += self.current_char
            self.advance()

        if result in KEYWORD_OPERATORS:
            return Token(KEYWORD_OPERATORS[result], result, start_line, start_column)
        elif result in RESERVED_KEYWORDS:
            return Token(TOKEN_KEYWORD, result, start_line, start_column)
        else:
            return Token(TOKEN_IDENTIFIER, result, start_line, start_column)

    def string_literal(self):
        result = ''
        start_line = self.line
        start_column = self.column
        quote_char = self.current_char 

        self.advance() # Consume opening quote

        while self.current_char is not None and self.current_char != quote_char:
            if self.current_char == '\\': # Handle escape sequences
                self.advance() # consume backslash
                if self.current_char is None:
                    self.error("Unterminated string literal (EOF after backslash).", line=start_line, column=start_column)
                elif self.current_char == quote_char:
                    result += quote_char
                elif self.current_char == '\\':
                    result += '\\'
                elif self.current_char == 'n': 
                    result += '\n'
                elif self.current_char == 't': 
                    result += '\t'
                else: # Unknown escape sequence, treat as literal backslash and char
                    result += '\\' + self.current_char
            else:
                result += self.current_char
            self.advance()

        if self.current_char is None: # Unterminated string
            self.error("Unterminated string literal (missing closing quote).", line=start_line, column=start_column)
        
        self.advance() # Consume closing quote
        return Token(TOKEN_STRING, result, start_line, start_column)

    def error(self, message="", line=None, column=None):
        """Raises a lexer error. Uses instance line/col if not provided."""
        current_line = line if line is not None else self.line
        current_column = column if column is not None else self.column
        full_message = f"Lexical error"
        if message:
            full_message += f": {message}"
        elif self.current_char: # Add current char info if no specific message
             full_message += f" on '{self.current_char}'"
        raise RxLexerError(full_message, line=current_line, column=current_column)

    def get_next_token(self):
        while self.current_char is not None:
            start_line = self.line
            start_column = self.column

            if self.current_char.isspace():
                if self.current_char == '\n':
                    self.advance()
                    return Token(TOKEN_NEWLINE, '\n', start_line, start_column)
                else:
                    self.skip_whitespace()
                    continue

            if self.current_char == '#':
                self.skip_comment()
                if self.current_char is None: break
                continue

            if self.current_char.isdigit():
                return self.number()

            if self.current_char.isalpha() or self.current_char == '_':
                return self.identifier()

            if self.current_char == '"' or self.current_char == "'":
                return self.string_literal()
            
            # --- Operators ---
            if self.current_char == '=':
                if self.peek() == '=': # ==
                    self.advance(); self.advance()
                    return Token(TOKEN_EE, '==', start_line, start_column)
                else: # =
                    self.advance()
                    return Token(TOKEN_ASSIGN, '=', start_line, start_column)
            
            if self.current_char == '!':
                if self.peek() == '=': # !=
                    self.advance(); self.advance()
                    return Token(TOKEN_NE, '!=', start_line, start_column)
                else:
                    # Standalone '!' is not a valid operator.
                    # Error will be caught by the fall-through unknown character handler,
                    # or we can make it more specific here.
                    # Let's make it specific to guide the user better.
                    char_at_error = self.current_char
                    # Advance before erroring to ensure progress if error is non-fatal (though ours are)
                    # self.advance() 
                    # Using start_line/start_column for the original position of '!'
                    self.error(f"Unknown character '{char_at_error}'. Did you mean '!=' for not equal?", line=start_line, column=start_column)
            
            if self.current_char == '<':
                if self.peek() == '=': # <=
                    self.advance(); self.advance()
                    return Token(TOKEN_LTE, '<=', start_line, start_column)
                else: # <
                    self.advance()
                    return Token(TOKEN_LT, '<', start_line, start_column)

            if self.current_char == '>':
                if self.peek() == '=': # >=
                    self.advance(); self.advance()
                    return Token(TOKEN_GTE, '>=', start_line, start_column)
                else: # >
                    self.advance()
                    return Token(TOKEN_GT, '>', start_line, start_column)

            if self.current_char == '+':
                if self.peek() == '=': # +=
                    self.advance(); self.advance()
                    return Token(TOKEN_PLUS_ASSIGN, '+=', start_line, start_column)
                else: # +
                    self.advance()
                    return Token(TOKEN_PLUS, '+', start_line, start_column)

            if self.current_char == '-':
                if self.peek() == '=': # -=
                    self.advance(); self.advance()
                    return Token(TOKEN_MINUS_ASSIGN, '-=', start_line, start_column)
                else: # -
                    self.advance()
                    return Token(TOKEN_MINUS, '-', start_line, start_column)

            if self.current_char == '*':
                if self.peek() == '=': # *=
                    self.advance(); self.advance()
                    return Token(TOKEN_MUL_ASSIGN, '*=', start_line, start_column)
                else: # *
                    self.advance()
                    return Token(TOKEN_MULTIPLY, '*', start_line, start_column)

            if self.current_char == '/':
                if self.peek() == '/':   
                    self.advance()       
                    if self.peek() == '=': 
                        self.advance()   
                        self.advance()   
                        return Token(TOKEN_INT_DIV_ASSIGN, '//=', start_line, start_column)
                    else: 
                        self.advance()   
                        return Token(TOKEN_INT_DIVIDE, '//', start_line, start_column)
                elif self.peek() == '=': 
                    self.advance(); self.advance()
                    return Token(TOKEN_DIV_ASSIGN, '/=', start_line, start_column)
                else: 
                    self.advance()
                    return Token(TOKEN_DIVIDE, '/', start_line, start_column)

            if self.current_char == '%':
                if self.peek() == '=': # %=
                    self.advance(); self.advance()
                    return Token(TOKEN_MOD_ASSIGN, '%=', start_line, start_column)
                else: # %
                    self.advance()
                    return Token(TOKEN_MODULO, '%', start_line, start_column)

            if self.current_char == '^':
                if self.peek() == '=': # ^=
                    self.advance(); self.advance()
                    return Token(TOKEN_POW_ASSIGN, '^=', start_line, start_column)
                else: # ^
                    self.advance()
                    return Token(TOKEN_POWER, '^', start_line, start_column)

            if self.current_char == '(':
                self.advance()
                return Token(TOKEN_LPAREN, '(', start_line, start_column)
            if self.current_char == ')':
                self.advance()
                return Token(TOKEN_RPAREN, ')', start_line, start_column)
            if self.current_char == ',':
                self.advance()
                return Token(TOKEN_COMMA, ',', start_line, start_column)
            
            # If char hasn't been handled by any rule above, it's unknown.
            unknown_char = self.current_char
            # self.advance() # Advance past the unknown character before erroring
            # Report error at the original position of the unknown character
            self.error(f"Unknown character: '{unknown_char}'", line=start_line, column=start_column)

        return Token(TOKEN_EOF, None, self.line, self.column) # Should be self.line, self.column at EOF

    def tokenize(self):
        tokens = []
        while True:
            token = self.get_next_token()
            tokens.append(token)
            if token.type == TOKEN_EOF:
                break
        return tokens



# rxscrite/interpreter.py

from .nodes import (
    NumberNode, UnaryOpNode, BinaryOpNode, VariableAssignNode,
    VariableAccessNode, FunctionCallNode, ProgramNode, IdentifierNode,
    StringNode, TruthNode, FalsityNode, EmptyNode, NoOpNode,
    CompoundAssignNode # Make sure this is imported
)
from .tokens import (
    TOKEN_PLUS, TOKEN_MINUS, TOKEN_MULTIPLY, TOKEN_DIVIDE, TOKEN_POWER,
    TOKEN_MODULO, TOKEN_INT_DIVIDE, # New arithmetic
    # Comparison tokens
    TOKEN_EE, TOKEN_NE, TOKEN_LT, TOKEN_GT, TOKEN_LTE, TOKEN_GTE,
    # Logical tokens
    TOKEN_LOGICAL_AND, TOKEN_LOGICAL_OR, TOKEN_LOGICAL_NOT,
    # Membership token (though full 'in' might need list support later)
    TOKEN_MEM_IN,
    # Compound assignment ops (used by CompoundAssignNode, op_token.type will be these)
    TOKEN_PLUS_ASSIGN, TOKEN_MINUS_ASSIGN, TOKEN_MUL_ASSIGN, TOKEN_DIV_ASSIGN,
    TOKEN_MOD_ASSIGN, TOKEN_POW_ASSIGN, TOKEN_INT_DIV_ASSIGN
)
from .errors import RxRuntimeError, RxNameError, RxTypeError, RxZeroDivisionError

class Interpreter:
    def __init__(self, filename="<stdin>"):
        self.filename = filename
        self.symbol_table = {} 
        self._setup_builtins()
        self.current_node_line = None # For error reporting context

    def _setup_builtins(self):
        self.symbol_table['display'] = BuiltInFunction('display', self._builtin_display)
        self.symbol_table['ask'] = BuiltInFunction('ask', self._builtin_ask)
        
        # These are Python values that TruthNode, FalsityNode, EmptyNode will produce.
        # They are not strictly needed in symbol_table if keywords always parse to specific nodes.
        self.symbol_table['Truth'] = True 
        self.symbol_table['Falsity'] = False
        self.symbol_table['Empty'] = None

    def _builtin_display(self, *args):
        output = " ".join(map(self._rx_str, args))
        print(output)
        return None 

    def _builtin_ask(self, *prompt_args):
        prompt_message = " ".join(map(self._rx_str, prompt_args)) if prompt_args else ""
        try:
            return input(prompt_message)
        except EOFError:
            return None # Represent EOF as Empty in RxScrite

    def _rx_str(self, value):
        if value is True: return "Truth"
        if value is False: return "Falsity"
        if value is None: return "Empty"
        return str(value)

    def visit(self, node):
        if node is None: # Should not happen with a proper AST, but safeguard
            return None
            
        if hasattr(node, 'line') and node.line is not None:
            self.current_node_line = node.line

        method_name = f'visit_{type(node).__name__}'
        visitor_method = getattr(self, method_name, self.generic_visit)
        # print(f"Visiting: {type(node).__name__} with {method_name} at line {self.current_node_line}") # Debug
        return visitor_method(node)

    def generic_visit(self, node):
        raise RxRuntimeError(f"No visit_{type(node).__name__} method defined", line=node.line, column=node.column)

    def visit_ProgramNode(self, node):
        result = None
        for statement_node in node.statements:
            result = self.visit(statement_node)
        return result # Value of last statement (mostly for REPL, scripts usually rely on display)

    def visit_NoOpNode(self, node):
        return None # No operation does nothing and returns nothing

    # --- Literal and Identifier Nodes ---
    def visit_NumberNode(self, node): return node.value
    def visit_StringNode(self, node): return node.value
    def visit_TruthNode(self, node): return True
    def visit_FalsityNode(self, node): return False
    def visit_EmptyNode(self, node): return None

    def visit_IdentifierNode(self, node): # Used for function names primarily now
        identifier_name = node.value
        if identifier_name in self.symbol_table:
            return self.symbol_table[identifier_name]
        raise RxNameError(f"Name '{identifier_name}' (from IdentifierNode) is not defined", line=node.line, column=node.column)

    def visit_VariableAccessNode(self, node):
        var_name = node.var_name_token.value
        if var_name in self.symbol_table:
            return self.symbol_table[var_name]
        raise RxNameError(f"Name '{var_name}' is not defined", line=node.line, column=node.column)

    # --- Assignment Nodes ---
    def visit_VariableAssignNode(self, node):
        var_name = node.var_name_token.value
        value = self.visit(node.value_node)
        self.symbol_table[var_name] = value
        return value # Assignment can be an expression in some languages

    def visit_CompoundAssignNode(self, node):
        var_name = node.var_name_token.value
        
        # Get current value of the variable
        if var_name not in self.symbol_table:
            raise RxNameError(f"Name '{var_name}' is not defined for compound assignment", line=node.var_name_token.line, column=node.var_name_token.column)
        current_value = self.symbol_table[var_name]
        
        # Get the value to apply from the right-hand side
        value_to_apply = self.visit(node.value_node)

        # Determine the base operation from the compound assignment token
        op_type = None
        if node.op_token.type == TOKEN_PLUS_ASSIGN: op_type = TOKEN_PLUS
        elif node.op_token.type == TOKEN_MINUS_ASSIGN: op_type = TOKEN_MINUS
        elif node.op_token.type == TOKEN_MUL_ASSIGN: op_type = TOKEN_MULTIPLY
        elif node.op_token.type == TOKEN_DIV_ASSIGN: op_type = TOKEN_DIVIDE
        elif node.op_token.type == TOKEN_MOD_ASSIGN: op_type = TOKEN_MODULO
        elif node.op_token.type == TOKEN_POW_ASSIGN: op_type = TOKEN_POWER
        elif node.op_token.type == TOKEN_INT_DIV_ASSIGN: op_type = TOKEN_INT_DIVIDE
        else:
            raise RxRuntimeError(f"Unknown compound assignment operator: {node.op_token.type}", line=node.op_token.line, column=node.op_token.column)

        # Perform the operation (reusing BinaryOpNode logic conceptually)
        # We need to handle types carefully here, similar to visit_BinaryOpNode
        if op_type == TOKEN_PLUS:
            if isinstance(current_value, (int, float)) and isinstance(value_to_apply, (int, float)):
                new_value = current_value + value_to_apply
            elif isinstance(current_value, str) or isinstance(value_to_apply, str): # String concatenation
                new_value = self._rx_str(current_value) + self._rx_str(value_to_apply)
            else:
                raise RxTypeError(f"Unsupported operand types for +: '{type(current_value).__name__}' and '{type(value_to_apply).__name__}'", line=node.op_token.line)
        elif isinstance(current_value, (int, float)) and isinstance(value_to_apply, (int, float)):
            if op_type == TOKEN_MINUS: new_value = current_value - value_to_apply
            elif op_type == TOKEN_MULTIPLY: new_value = current_value * value_to_apply
            elif op_type == TOKEN_DIVIDE:
                if value_to_apply == 0: raise RxZeroDivisionError("Division by zero", line=node.op_token.line)
                new_value = current_value / value_to_apply
            elif op_type == TOKEN_MODULO:
                if value_to_apply == 0: raise RxZeroDivisionError("Modulo by zero", line=node.op_token.line)
                new_value = current_value % value_to_apply
            elif op_type == TOKEN_POWER: new_value = current_value ** value_to_apply
            elif op_type == TOKEN_INT_DIVIDE:
                if value_to_apply == 0: raise RxZeroDivisionError("Integer division by zero", line=node.op_token.line)
                new_value = current_value // value_to_apply
            else: # Should not happen due to earlier check
                raise RxRuntimeError(f"Unhandled numeric operation for compound assignment: {op_type}", line=node.op_token.line)
        else:
            raise RxTypeError(f"Compound assignment operator '{node.op_token.value}' not supported for types '{type(current_value).__name__}' and '{type(value_to_apply).__name__}'", line=node.op_token.line)

        self.symbol_table[var_name] = new_value
        return new_value

    # --- Operator Nodes ---
    def visit_UnaryOpNode(self, node):
        operand_value = self.visit(node.node)

        if node.op_token.type == TOKEN_MINUS:
            if not isinstance(operand_value, (int, float)):
                raise RxTypeError(f"Unary '-' not supported for type {type(operand_value).__name__}", line=node.line)
            return -operand_value
        elif node.op_token.type == TOKEN_PLUS: # Unary plus
            if not isinstance(operand_value, (int, float)):
                raise RxTypeError(f"Unary '+' not supported for type {type(operand_value).__name__}", line=node.line)
            return +operand_value
        elif node.op_token.type == TOKEN_LOGICAL_NOT: # 'not' operator
            # In Python, 'not' works on any value (truthy/falsy).
            # We'll mimic this: 0, 0.0, "", Empty, Falsity are falsy. Others are truthy.
            if operand_value is False or operand_value is None or operand_value == 0 or operand_value == "":
                return True # not Falsy -> Truth
            else:
                return False # not Truthy -> Falsity
        
        raise RxRuntimeError(f"Unknown unary operator: {node.op_token.type}", line=node.line)

    def visit_BinaryOpNode(self, node):
        left_val = self.visit(node.left_node)
        
        # Short-circuiting for 'and' and 'or'
        if node.op_token.type == TOKEN_LOGICAL_AND:
            # If left_val is falsy, result is left_val, don't eval right_val
            is_left_falsy = left_val is False or left_val is None or left_val == 0 or left_val == ""
            if is_left_falsy:
                return left_val # Python's 'and' returns the first falsy value or the last value
            else:
                return self.visit(node.right_node) # Return result of right side

        if node.op_token.type == TOKEN_LOGICAL_OR:
            # If left_val is truthy, result is left_val, don't eval right_val
            is_left_truthy = not (left_val is False or left_val is None or left_val == 0 or left_val == "")
            if is_left_truthy:
                return left_val # Python's 'or' returns the first truthy value or the last value
            else:
                return self.visit(node.right_node) # Return result of right side

        # For other binary ops, evaluate right operand
        right_val = self.visit(node.right_node)
        op_type = node.op_token.type

        # Arithmetic and String Concatenation
        if op_type == TOKEN_PLUS:
            if isinstance(left_val, (int, float)) and isinstance(right_val, (int, float)):
                return left_val + right_val
            elif isinstance(left_val, str) or isinstance(right_val, str): # String concatenation
                return self._rx_str(left_val) + self._rx_str(right_val)
            else:
                raise RxTypeError(f"Unsupported operand types for +: '{type(left_val).__name__}' and '{type(right_val).__name__}'", line=node.op_token.line)

        # Numeric Arithmetic Operations (excluding + which is handled above)
        if isinstance(left_val, (int, float)) and isinstance(right_val, (int, float)):
            if op_type == TOKEN_MINUS: return left_val - right_val
            elif op_type == TOKEN_MULTIPLY: return left_val * right_val
            elif op_type == TOKEN_DIVIDE:
                if right_val == 0: raise RxZeroDivisionError("Division by zero", line=node.op_token.line)
                return left_val / right_val
            elif op_type == TOKEN_POWER: return left_val ** right_val
            elif op_type == TOKEN_MODULO:
                if right_val == 0: raise RxZeroDivisionError("Modulo by zero", line=node.op_token.line)
                return left_val % right_val
            elif op_type == TOKEN_INT_DIVIDE:
                if right_val == 0: raise RxZeroDivisionError("Integer division by zero", line=node.op_token.line)
                return left_val // right_val
        
        # Comparison Operations (can work on various types, like Python)
        if op_type == TOKEN_EE: return left_val == right_val # Python's == handles type flexibility
        elif op_type == TOKEN_NE: return left_val != right_val # Python's !=
        # For <, <=, >, >=, Python raises TypeError if types are incompatible and not ordered.
        # We'll rely on Python's comparison behavior here.
        try:
            if op_type == TOKEN_LT: return left_val < right_val
            elif op_type == TOKEN_LTE: return left_val <= right_val
            elif op_type == TOKEN_GT: return left_val > right_val
            elif op_type == TOKEN_GTE: return left_val >= right_val
        except TypeError:
             raise RxTypeError(f"Comparison '{node.op_token.value}' not supported between instances of '{type(left_val).__name__}' and '{type(right_val).__name__}'", line=node.op_token.line)

        # Membership 'in' (rudimentary for now, assumes right_val is a string if left_val is char/string)
        # Full 'in' needs iterable support for lists, etc.
        if op_type == TOKEN_MEM_IN:
            if isinstance(right_val, str) and isinstance(left_val, str):
                return left_val in right_val
            # Add more sophisticated 'in' for lists/tuples/dicts when they are implemented
            raise RxTypeError(f"'in' operator not fully supported for types '{type(left_val).__name__}' and '{type(right_val).__name__}' yet.", line=node.op_token.line)

        # If we fall through, it's an unhandled binary op for the given types
        raise RxTypeError(f"Unsupported operand types for binary operator '{node.op_token.value}': '{type(left_val).__name__}' and '{type(right_val).__name__}'", line=node.op_token.line)


    def visit_FunctionCallNode(self, node):
        callable_obj = self.visit(node.func_name_node)

        if not isinstance(callable_obj, BuiltInFunction): # Later: add UserDefinedFunction
            func_name_str = node.func_name_node.value if hasattr(node.func_name_node, 'value') else 'unknown function'
            raise RxTypeError(f"'{func_name_str}' ({type(callable_obj).__name__}) is not a function", line=node.line)

        args = [self.visit(arg_node) for arg_node in node.arg_nodes]
        
        original_line = self.current_node_line
        if hasattr(node, 'line') and node.line is not None:
            self.current_node_line = node.line
        try:
            return callable_obj.execute(args, self)
        except RxError: raise
        except Exception as e:
            raise RxRuntimeError(f"Error during built-in function '{callable_obj.name}': {e}", line=self.current_node_line)
        finally:
            self.current_node_line = original_line

class BuiltInFunction:
    def __init__(self, name, python_callable=None):
        self.name = name
        self.python_callable = python_callable

    def execute(self, args, interpreter_instance): # interpreter_instance can be used for context
        if self.python_callable:
            try:
                return self.python_callable(*args)
            except TypeError as e: # Catch arity errors or other Python TypeErrors from the builtin
                # Try to provide a more RxScrite-like error message
                raise RxTypeError(f"Error calling built-in '{self.name}': {e}", line=interpreter_instance.current_node_line)
        else:
            raise NotImplementedError(f"Built-in function '{self.name}' is not implemented.")

    def __repr__(self):
        return f"<BuiltInFunction:{self.name}>"



# rxscrite/__init__.py

"""
RxScrite Programming Language
-----------------------------

An educational programming language built with Python.
"""

# You can choose to expose key components here if you want them to be
# accessible directly from `import rxscrite` or `from rxscrite import ...`
# For example:
# from .lexer import Lexer, LexerError
# from .parser import Parser, ParserError
# from .interpreter import Interpreter, RuntimeError
# from .tokens import Token

# For now, we'll keep it simple. The CLI will use relative imports.

VERSION = "0.1.0"


from setuptools import setup, find_packages

# Define your version here.
VERSION = "0.1.0"

# Try to read README.md for the long description
try:
    with open("README.md", "r", encoding="utf-8") as fh:
        long_description = fh.read()
except FileNotFoundError:
    long_description = "RxScrite: An educational programming language built with Python."

setup(
    name="rxscrite", # <<< CHECK PYPI.ORG FOR UNIQUENESS! Change if "rxscrite" is taken.
    version=VERSION,
    author="Rx MHA",  # <<< CHANGE THIS to your actual name
    author_email="rxsocialmedia1@gmail.com",  # <<< CHANGE THIS to your actual email
    description="RxScrite: An educational programming language",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/rxmha125/rxscrite_project",  # <<< CHANGE THIS to your project's URL (Optional)
    
    # This will find your 'rxscrite' package directory containing your .py files
    packages=find_packages(where=".", include=["rxscrite", "rxscrite.*"]),
    
    classifiers=[
        "License :: OSI Approved :: MIT License",  # Ensure you have a matching LICENSE file
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Operating System :: OS Independent",
        "Topic :: Education",
        "Topic :: Software Development :: Interpreters",
        "Development Status :: 3 - Alpha", 
    ],
    python_requires=">=3.8", # Minimum Python version your project supports
    
    # This creates the command-line script `rxscrite`
    # So after 'pip install .', you can type 'rxscrite yourfile.rx'
    entry_points={
        "console_scripts": [
            "rxscrite=rxscrite.cli:main", 
        ],
    },
    # If your project had other Python package dependencies, you would list them here
    # install_requires=[], # No external dependencies for RxScrite so far
)


# rxscrite/cli.py

import argparse
import sys
import os # For checking if file exists

# Assuming your package structure is correct, these relative imports should work
# when running with `python -m rxscrite.cli ...` or as an installed package.
from .lexer import Lexer
from .parser import Parser
from .interpreter import Interpreter
from .errors import RxError # Base error class for RxScrite
# from . import VERSION # If you have VERSION in rxscrite/__init__.py

def run_rxscrite_file(filepath):
    """
    Reads, lexes, parses, and interprets an RxScrite file.
    """
    if not os.path.exists(filepath):
        print(f"Error: File not found: {filepath}", file=sys.stderr)
        sys.exit(1)
    
    if not filepath.endswith(".rx"):
        print(f"Error: RxScrite files must have a '.rx' extension. Provided: {filepath}", file=sys.stderr)
        sys.exit(1)

    try:
        with open(filepath, 'r', encoding='utf-8') as f:
            source_code = f.read()
        
        if not source_code.strip():
            print(f"File '{filepath}' is empty. Nothing to execute.")
            return

        # 1. Lexer
        lexer = Lexer(source_code, filepath)
        tokens = lexer.tokenize()
        
        # 2. Parser
        # NOTE: The script will likely fail here or in the interpreter
        # until the parser and interpreter are updated for the new tokens/grammar.
        parser = Parser(tokens, filepath)
        ast = parser.parse()
        # print("--- AST ---") # Uncomment to see AST if parser works
        # if ast and hasattr(ast, 'statements'):
        #     for i, stmt in enumerate(ast.statements):
        #         print(f"Stmt {i}: {stmt}")
        # print("--- End AST ---\n")


        # 3. Interpreter
        interpreter = Interpreter(filepath)
        result = interpreter.visit(ast)
        
        # RxScrite's `display()` handles output.
        # The result of the script itself (last expression) might be None
        # or the value of the last expression if not a display call.
        # We don't explicitly print `result` here unless it's meaningful
        # for the language's design (e.g., for a REPL).
        # if result is not None:
        #     print(interpreter._rx_str(result))

    except RxError as e:
        # This catches RxLexerError, RxParserError, RxRuntimeError, etc.
        print(str(e), file=sys.stderr)
        sys.exit(1)
    except FileNotFoundError: # Should be caught by os.path.exists, but as fallback
        print(f"Error: Source file '{filepath}' not found.", file=sys.stderr)
        sys.exit(1)
    except Exception as e:
        # Catch any other unexpected Python errors during execution
        print(f"An unexpected Python error occurred: {e}", file=sys.stderr)
        # For deeper debugging of Python errors:
        # import traceback
        # traceback.print_exc()
        sys.exit(1)

def main():
    parser_arg = argparse.ArgumentParser(description="RxScrite Programming Language Interpreter")
    parser_arg.add_argument(
        "filepath",
        type=str,
        help="Path to the .rx RxScrite file to execute."
    )
    # Example for adding a version argument later, if VERSION is defined in __init__.py
    # parser_arg.add_argument(
    #     "-v", "--version", action="version",
    #     version=f"RxScrite {VERSION if 'VERSION' in globals() else 'unknown'}"
    # )

    args = parser_arg.parse_args()
    
    run_rxscrite_file(args.filepath)

if __name__ == '__main__':
    # This allows running `python rxscrite/cli.py yourfile.rx` directly
    # from the project root, for development.
    main()
